////////////////////////////////////////////////////////////////////////////////
//	BaseViewer.h:	The base class for file viewing classes
//
//	This is the base class for all file viewing classes.  It contains definitions
//	of several basic data types used to extract binary data from the input files,
//	along with the basic routines that use these data types and some basic file
//	open/read/close processing common to any file viewer.
//
//	NOTE: This is assumed to be an open-source project, so individual method
//	notes appear before each method in the CPP file rather than here in the
//	.h file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASEVIEWER_H_INCLUDED_)
#define AFX_BASEVIEWER_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CBaseViewer : public CObject  
{
public:
					CBaseViewer		();
	virtual			~CBaseViewer	();

	virtual void	ViewFile			(char *);
			void	GetOutputFileName	(char *	szFileName);

protected:
	bool ViewChunkHeader (bool bIsFileHeader);

	typedef struct Point2f		// a two value point
	{
		float	fValue [2];
	};

	typedef struct Vector3f		// a three dimentional vector (x, y, z)
	{
		float	fValue [3];
	};

	typedef struct Matrix3f		// a 3 X 3 dimentional vector
	{
		float	fValue [9];
	};

	typedef struct Quaternion	// a quaternion
	{
		float	fValue[4];
	};

	virtual BOOL	OpenTheFiles	();
			void	CloseTheFiles	();
			BOOL	ReadInput		(void*				lpBuffer,
									 DWORD				dwLength);
			BOOL	ReadWORD		(WORD*				lpUINT);
			BOOL	ReadDWORD		(DWORD*				lpDWORD);
			BOOL	ReadPoint2f		(struct Point2f *	ThePoint);
			BOOL	ReadVector3f	(struct Vector3f*	TheVector);
			BOOL	ReadMatrix3f	(struct Matrix3f*	TheMatrix);
			BOOL	ReadQuaternion	(struct Quaternion*	TheQuaternion);
			BOOL	ReadFloat		(float*				lpFloat);
			BOOL	WriteOutput		(char*				lpBuffer,
									 DWORD				dwLength);
//
//			Values used by all file viewers
//
	HANDLE	m_hInputFile;					// input file
	HANDLE	m_hOutputFile;					// output file
	DWORD	m_dwFileSize;					// total file size
	DWORD	m_dwTotalFileSize;				// the size of the entire file
	DWORD	m_dwFileSizeRead;				// the amount of the file last read
	DWORD	m_dwChunkSize;					// the size of the current chunk
	DWORD	m_dwVersion;					// the current chunk's version ID
	DWORD	m_dwTotalChunkCount;			// the count of all chunks read so far
	DWORD	m_dwChunksFound;				// the number of chunks found
	char	m_InputBuffer		[10000];	// general input file buffer
	char	m_OutputBuffer		[10000];	// general output file buffer
	char	m_szInputFileName	[MAX_PATH];	// input file name
	char	m_szOutputFileName	[MAX_PATH];	// output file name
	char	m_szMessage			[1024];		// general error message buffer.
	char	m_szFileType		[5];		// name of the type of the file.
	char	m_szLastChunkRead	[5];		// name of the last chunk read

};

#endif // !defined(AFX_BASEVIEWER_H_INCLUDED_)
////////////////////////////////////////////////////////////////////////////////
