////////////////////////////////////////////////////////////////////////////////
//	HexViewer.cpp:	The class that produces hex output of a file
//
//	This class creates a hexidecimal dump of the input.  It can read any input
//	file in any binary or text format.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "FileViewer.h"
#include "HexViewer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CHexViewer::CHexViewer()
{

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
}

CHexViewer::~CHexViewer()
{

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Provides the main processing function of viewing any file.
//
//   COMMENTS:
//
//	This function provides the main function of viewing any file as a hex
//	dump showing the contents as both hex and character data.
//
//	This function will not return until the entire file has been processed.
//	
//	  szFileName		The name of the file to view
//
//    RETURN VALUES:
//        None.  This class must handle all errors.
//
void CHexViewer::ViewFile(char* szFileName)
{
	BOOL	bDirty				= FALSE;
	DWORD	dwFileSize;
	DWORD	dwDefaultChunkSize	= 1024;
	DWORD	dwChunkSize			= 0;
	DWORD	dwChunkRemaining	= 0;
	DWORD	nFileAddr			= 0;
	int		nCharPtr			= 0;
	int		nLinePosn			= 0;
	BYTE	nChar;
	int		nCharPosn;

	char szDumpText [100];
	char szFmtText  [10];
	
//
//			Open the input and output files
//
	strcpy (m_szInputFileName, szFileName);
	if (! OpenTheFiles ())
	{
	   	MessageBox (NULL, "File open failed, returning from IMViewer", NULL, MB_OK );
		return;
	}
	dwFileSize = GetFileSize (m_hInputFile, NULL);
	if (dwFileSize == 0xFFFFFFFF)
	{
	    MessageBox( NULL, "Cannot read file size, hex file view failed.", NULL, MB_OK );
		CloseTheFiles ();
		return;
	}
//
//			Read through the file a little at a time dumping the formated contents
//			the output file
//
	while (dwFileSize > 0)
	{
		if (dwChunkRemaining < 1)
		{
//
//			Out of data, go read some more from the input file
//
			if (dwFileSize > dwDefaultChunkSize)
				dwChunkSize = dwDefaultChunkSize;
			else
				dwChunkSize = dwFileSize;
			if (! ReadInput (m_InputBuffer, dwChunkSize))
			{
				CloseTheFiles ();
				return;
			}
			dwFileSize -= dwChunkSize;
			dwChunkRemaining = dwChunkSize;
			nCharPtr = 0;
		}
//
//			Dump the contents of the current buffer
//
		while (nCharPtr < (int)dwChunkSize )
		{
				// If we have filled up a 16 char line, print it

			if (nLinePosn >= 16)
			{
				if (! WriteOutput (szDumpText, strlen(szDumpText)))
				{
					CloseTheFiles ();
					return;
				}
				
				bDirty = FALSE;
				nLinePosn = 0;
			}
				// If we are starting a new line, initialize it

			if (nLinePosn == 0)
			{
				// NOTE:  the following line is fairly long, so if you can't
				//        see the end of it. scroll the screen to the right.

				wsprintf (szDumpText,
				"(%010u) %08X                                                     |                |\r\n\0", nFileAddr, nFileAddr);
			}
				// Get the next character and put its hex value into the dump text
				// output string.  Add one character to the second 8 characters of
				// each line to visually break the dump into two 8-byte sections

			nChar = m_InputBuffer[nCharPtr++];
			nFileAddr++;
			wsprintf (szFmtText, "%02X", nChar);
			nCharPosn = 23 + (nLinePosn * 3);
			if (nLinePosn >= 8)
				nCharPosn++;
			szDumpText [nCharPosn]     = szFmtText [0];
			szDumpText [nCharPosn + 1] = szFmtText [1];

				// If the character is printable, insert it into the character
				// portion of the dump, otherwise, insert a period.

			if (isprint (nChar))
				szFmtText[0] = nChar;
			else
				szFmtText[0] = '.';
			nCharPosn = 75 + nLinePosn++;
			szDumpText [nCharPosn]     = szFmtText [0];
			bDirty = TRUE;
			dwChunkRemaining--;
		}
	
	}

	if (bDirty)
	{
		if (! WriteOutput (szDumpText, strlen(szDumpText)))
		{
			CloseTheFiles ();
			return;
		}
	}

	CloseTheFiles ();

	return;
}
////////////////////////////////////////////////////////////////////////////////
